/**
 * 格式化分类列表
 * @param {Array} arr 分类数据(包含Children的二级树形数据)
 * @param {String} parentIdName 一级节点Id名称,作为tree的nodeKeyId
 * @param {String} itemIdName 二级节点Id名称
 * @returns Array
 */
export const formatArr = (arr,parentIdName='SaDocClassifyId',itemIdName='SaDocId') => {
  let tempArr = arr.map((item) => {
    item.label = item.ClassifyName
    item.nodeKeyId = item[parentIdName] + ''
    item.children = item.Children ? item.Children.map((i) => {
      i.label = i.Title
      i.nodeKeyId = 'children_' + i[itemIdName]
      i.parentId = item[parentIdName]
      if(item.CoverImg){
        i.CoverImg = item.CoverImg
      }
      return i
    }) : []
    return item
  })
  return tempArr
}

/* 文档渲染相关 */

/**
 * 获取段落的纯文本
 * @param {Object} section 段落数据 
 * @returns String
 */
export const formatFileContent = (section) => {
  const tempdiv = document.createElement('div')
  tempdiv.innerHTML = section.Content
  return tempdiv.innerText
}
/**
 * 从标签数组中筛选出对应分类的标签
 * @param {Array} LabelList 段落已打标签
 * @param {String} type 分类标签名
 * @returns Array
 */
export const formatFileLabel = (LabelList, type) => {
  const labelList = LabelList.filter(label => {
    return label[type] === 1
  })
  return labelList
}
/**
 * 判断selectArr与arr是否有重合的项
 * @param {Array} selectArr 
 * @param {*} arr 
 * @returns Boolean
 */
export const checkSelectedLabel = (selectArr, arr) => {
  const selectArrIds = selectArr.map(i => {
    return i.LabelId
  })
  const arrIds = arr.map(i => {
    return i.LabelId
  })
  if(arrIds.length===0||selectArrIds.length === 0) return false
  const concatIds = new Set(selectArrIds.concat(arrIds))
  return !(concatIds.size === (arrIds.length + selectArrIds.length))
}
/**
 * 获取文档的纯文本
 * @param {Object} file 
 * @returns 
 */
export const formatFile = (file)=>{
  file.SectionList = file.SectionList.map(section=>{
    section.innerText = formatFileContent(section)
    return section
  })
  return file
}
//格式化语义分析文档，格式化标签信息
/*
  具体操作如下:遍历file.SectionList
    1.获取段落纯文本
    2.根据LabelList,获取isThisLabelArr,isHistoryLabelArr,isOtherLabelArr三个数组
    3.若获取isThisLabelArr有值,则ishasThisLabel为true
    4.将2中的3个数组与selectArr比较,若有一个重合,则isSelectLabel为true
    (isThisLabelArr 该语义分析内该段落被打的当前标签 isHistoryLabelArr 该段落被操作者打的历史标签 isOtherLabelArr该段落被打的其他人标签)
 */
/**
 * 格式化语义分析文档，格式化标签信息
 * @param {Object} file 
 * @param {Array} selectArr 
 * @returns Object
 */
export const formatSemanticFile = (file,selectArr)=>{
  let temp = _.cloneDeep(file)
  temp.SectionList = temp.SectionList.map(item => {
    const {LabelList} = item
    if(!LabelList){
      item.LabelList = []
    }
    item.innerText = formatFileContent(item)
    item.isThisLabelArr = formatFileLabel(LabelList || [], 'IsThis')
    item.isHistoryLabelArr = formatFileLabel(LabelList || [], 'IsHistory')
    item.isOtherLabelArr = formatFileLabel(LabelList || [], 'IsOther')
    item.ishasThisLabel = item.isThisLabelArr.length ? true : false
    item.isSelectLabel = checkSelectedLabel(selectArr, item.isHistoryLabelArr) ||
      checkSelectedLabel(selectArr, item.isOtherLabelArr)||checkSelectedLabel(selectArr,item.isThisLabelArr)
    return item
  })
  return temp
}

/**
 * 格式化语义分析数据,转换成接口格式
 * @param {Object} file 
 * @returns Object
 */
export const formatCompareListItem = (file)=>{
  let temp = {}
  temp.SaDocId = file.DocId
  temp.SectionList = file.SectionList.map(item=>{
    const {isThisLabelArr} = item
    const LabelIds = isThisLabelArr?isThisLabelArr.map(l=>{
      return l.LabelId
    }):[]
    return {//所有的IsPart为0,Content为空,LabelIds为isThisLabelArr[].LabelId
      SaDocSectionId:item.SectionId,
      IsPart:0,Content:'',LabelIds
    }
  })
  return temp
}
/**
 * 获取默认的LabelList 用于内容提取展示
 * @param {Number} docNum 有多少篇文章
 * @returns arr
 */
export const getDefaultLabelList = (docNum)=>{
  let temp = {
    SaLabelId:1,
    LabelName:'暂无关键词',
    DocList:[]
  }
  temp.DocList = Array.from({length:docNum}).map((_,index)=>{
    return {
      SaDocId:index+1,
      SectionList:[]
    }
  })
  return [temp]
}

/*------*/

/* 搜索高亮相关 */
//参考：https://juejin.cn/post/7199438741533376573
//https://developer.mozilla.org/zh-CN/docs/Web/API/CSS_Custom_Highlight_API
export const getTextNodes = (dom)=>{
  const treeWalker = document.createTreeWalker(dom, NodeFilter.SHOW_TEXT);
  const allTextNodes = [];
  let currentNode = treeWalker.nextNode();
  while (currentNode) {
    allTextNodes.push(currentNode);
    currentNode = treeWalker.nextNode();
  }
  return allTextNodes
}
export const highlightWord = (allTextNodes, searchText) => {
  const str = searchText.toLowerCase()
  const ranges = allTextNodes
    .map((el) => {
      return {
        el,
        text: el.textContent.toLowerCase()
      };
    })
    .map(({
      text,
      el
    }) => {
      const indices = [];
      let startPos = 0;
      while (startPos < text.length) {
        const index = text.indexOf(str, startPos);
        if (index === -1) break;
        indices.push(index);
        startPos = index + str.length;
      }
      // 根据搜索词的位置创建选区
      return indices.map((index) => {
        const range = new Range();
        range.setStart(el, index);
        range.setEnd(el, index + str.length);
        return range;
      });
    })
  // 创建高亮对象
  const searchResultsHighlight = new Highlight(...ranges.flat());
  // 注册高亮
  CSS.highlights.set("search-results", searchResultsHighlight);
}